//
//  AppDelegate.m
//  SerialPort
//
//  Created by Yoshiaki Koyama on 2015/08/09.
//  Copyright (c) 2015年 Yoshiaki Koyama. All rights reserved.
//
#import "AppDelegate.h"
#import <IOKit/serial/IOSerialKeys.h>
#import <IOKit/serial/ioss.h>

@interface AppDelegate ()

@property (unsafe_unretained) IBOutlet NSWindow *window;
@end

@implementation AppDelegate

/*
 
 */
- (void)windowWillClose:(NSNotification *)notification {    
    [NSApp terminate:self];
}

/*
 
 */
- (void)applicationDidFinishLaunching:(NSNotification *)aNotification {
    // Insert code here to initialize your application
    
    // 関数呼び出し
    [self bsdPathFromDevice];
}

/*
 
 */
- (void)applicationWillTerminate:(NSNotification *)aNotification {
    // Insert code here to tear down your application
}

/*
 IOKit からデバイス名を取り出す
 */
-(void)bsdPathFromDevice {
    
    CFMutableDictionaryRef matchingDict = NULL;
    
    // kIOSerialBSDServiceValue にマッチした IOService の CFMutableDictionaryRef を作成
    matchingDict = IOServiceMatching(kIOSerialBSDServiceValue);
    CFRetain(matchingDict); // Need to use it twice
    
    // matchingDict に キー:CFSTR(kIOSerialBSDTypeKey) 値:CFSTR(kIOSerialBSDAllTypes) を追加
    CFDictionaryAddValue(matchingDict, CFSTR(kIOSerialBSDTypeKey), CFSTR(kIOSerialBSDAllTypes));
    
    // portIterator に matchingDict で指定した IOService object をセット
    io_iterator_t portIterator = 0;
    kern_return_t err = IOServiceGetMatchingServices(kIOMasterPortDefault, matchingDict, &portIterator);
    CFRelease(matchingDict);
    
    if (err) return;
    
    // portIterator からシリアルポート eachPort を取り出す
    io_object_t eachPort = 0;
    while ((eachPort = IOIteratorNext(portIterator)))
    {
        // デバイス名を取得
        NSString *calloutPath = [AppDelegate bsdCalloutPathFromDevice:eachPort];  // /dev/cu.xxxxx
        NSString *dialinPath = [AppDelegate bsdDialinPathFromDevice:eachPort];    // /dev/tty.xxxx
        
        NSLog(@"%@\n", calloutPath);
        NSLog(@"%@\n", dialinPath);
        
        IOObjectRelease(eachPort);
    }
    IOObjectRelease(portIterator);
    
}

/*
 シリアルポートのデバイス名を取り出す関数
 */
+ (NSString *)stringPropertyOf:(io_object_t)aDevice forIOSerialKey:(NSString *)key
{
    
    // ここが IOKit からシリアルポートのデバイス名を取り出している箇所
    CFStringRef string = (CFStringRef)IORegistryEntryCreateCFProperty(aDevice,
                                                                      (__bridge CFStringRef)key,
                                                                      kCFAllocatorDefault,
                                                                      0);
    return (__bridge_transfer NSString *)string;
}


/*
 
 /Applications/Xcode.app/Contents/Developer/Platforms/MacOSX.platform/Developer/SDKs/MacOSX10.10.sdk/System/Library/Frameworks/IOKit.framework/Versions/A/Headers/serial/IOSerialKeys.h

    IOSerialKeys.h
    
    #define kIOCalloutDeviceKey		"IOCalloutDevice"
    #define kIODialinDeviceKey		"IODialinDevice"

 */

/*
 kIOCalloutDeviceKey: @"/dev/cu.xxxxx" を返す
 */
+ (NSString *)bsdCalloutPathFromDevice:(io_object_t)aDevice
{
    return [self stringPropertyOf:aDevice forIOSerialKey:(NSString*)CFSTR(kIOCalloutDeviceKey)];
}

/*
 kIODialinDeviceKey: @"dev/tty.xxxxx" を返す
 */
+ (NSString *)bsdDialinPathFromDevice:(io_object_t)aDevice
{
    return [self stringPropertyOf:aDevice forIOSerialKey:(NSString*)CFSTR(kIODialinDeviceKey)];
}

@end
